@@ -1,5 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set shiftwidth=2 tabstop=2 autoindent cindent expandtab: */
 /* jshint node:true */
 /* globals cat, cd, echo, ls */
 'use strict';
@@ -8,7 +6,6 @@ require('shelljs/make');
 
 var builder = require('./builder');
 var fs = require('fs');
-var path = require('path');
 
 var errors = 0;
 
