@@ -1,5 +1,3 @@
-/* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set shiftwidth=2 tabstop=2 autoindent cindent expandtab: */
 /* Copyright 2012 Mozilla Foundation
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,7 +12,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-/* globals PDFJS, combineUrl, StatTimer, Promise */
+/* globals PDFJS, pdfjsSharedUtil */
 
 'use strict';
 
@@ -24,74 +22,203 @@ var PDF_TO_CSS_UNITS = 96.0 / 72.0;
 /**
  * @class
  */
-var NullTextLayerBuilder = (function NullTextLayerBuilderClosure() {
-  /**
-   * @constructs NullTextLayerBuilder
-   */
-  function NullTextLayerBuilder() {}
+var LinkServiceMock = (function LinkServiceMockClosure() {
+  function LinkServiceMock() {}
 
-  NullTextLayerBuilder.prototype = {
-    beginLayout: function NullTextLayerBuilder_BeginLayout() {},
-    endLayout: function NullTextLayerBuilder_EndLayout() {},
-    appendText: function NullTextLayerBuilder_AppendText() {}
+  LinkServiceMock.prototype = {
+    navigateTo: function (dest) {},
+
+    getDestinationHash: function (dest) {
+      return '#';
+    },
+
+    getAnchorUrl: function (hash) {
+      return '#';
+    },
+
+    executeNamedAction: function (action) {}
   };
 
-  return NullTextLayerBuilder;
+  return LinkServiceMock;
 })();
 
 /**
  * @class
  */
-var SimpleTextLayerBuilder = (function SimpleTextLayerBuilderClosure() {
-  /**
-   * @constructs SimpleTextLayerBuilder
-   */
-  function SimpleTextLayerBuilder(ctx, viewport) {
-    this.ctx = ctx;
-    this.viewport = viewport;
-    this.textCounter = 0;
+var rasterizeTextLayer = (function rasterizeTextLayerClosure() {
+  var SVG_NS = 'http://www.w3.org/2000/svg';
+
+  var textLayerStylePromise = null;
+  function getTextLayerStyle() {
+    if (textLayerStylePromise) {
+      return textLayerStylePromise;
+    }
+    textLayerStylePromise = new Promise(function (resolve) {
+      var xhr = new XMLHttpRequest();
+      xhr.open('GET', './text_layer_test.css');
+      xhr.onload = function () {
+        resolve(xhr.responseText);
+      };
+      xhr.send(null);
+    });
+    return textLayerStylePromise;
   }
 
-  SimpleTextLayerBuilder.prototype = {
-    appendText: function SimpleTextLayerBuilder_AppendText(geom, styles) {
-      var style = styles[geom.fontName];
-      var ctx = this.ctx, viewport = this.viewport;
-      var tx = PDFJS.Util.transform(this.viewport.transform, geom.transform);
-      var angle = Math.atan2(tx[1], tx[0]);
-      var fontHeight = Math.sqrt((tx[2] * tx[2]) + (tx[3] * tx[3]));
-      var fontAscent = (style.ascent ? style.ascent * fontHeight :
-        (style.descent ? (1 + style.descent) * fontHeight : fontHeight));
+  function rasterizeTextLayer(ctx, viewport, textContent) {
+    return new Promise(function (resolve) {
+      // Building SVG with size of the viewport.
+      var svg = document.createElementNS(SVG_NS, 'svg:svg');
+      svg.setAttribute('width', viewport.width + 'px');
+      svg.setAttribute('height', viewport.height + 'px');
+      // items are transformed to have 1px font size
+      svg.setAttribute('font-size', 1);
 
-      ctx.save();
-      ctx.beginPath();
-      ctx.strokeStyle = 'red';
-      ctx.fillStyle = 'yellow';
-      ctx.translate(tx[4] + (fontAscent * Math.sin(angle)),
-                    tx[5] - (fontAscent * Math.cos(angle)));
-      ctx.rotate(angle);
-      ctx.rect(0, 0, geom.width * viewport.scale, geom.height * viewport.scale);
-      ctx.stroke();
-      ctx.fill();
-      ctx.restore();
-      ctx.font = fontHeight + 'px ' + style.fontFamily;
-      ctx.fillStyle = 'black';
-      ctx.fillText(geom.str, tx[4], tx[5]);
+      // Adding element to host our HTML (style + text layer div).
+      var foreignObject = document.createElementNS(SVG_NS, 'svg:foreignObject');
+      foreignObject.setAttribute('x', '0');
+      foreignObject.setAttribute('y', '0');
+      foreignObject.setAttribute('width', viewport.width + 'px');
+      foreignObject.setAttribute('height', viewport.height + 'px');
+      var style = document.createElement('style');
+      var stylePromise = getTextLayerStyle();
+      foreignObject.appendChild(style);
+      var div = document.createElement('div');
+      div.className = 'textLayer';
+      foreignObject.appendChild(div);
 
-      this.textCounter++;
-    },
+      // Rendering text layer as HTML.
+      var task = PDFJS.renderTextLayer({
+        textContent: textContent,
+        container: div,
+        viewport: viewport
+      });
+      Promise.all([stylePromise, task.promise]).then(function (results) {
+        style.textContent = results[0];
+        svg.appendChild(foreignObject);
 
-    setTextContent:
-        function SimpleTextLayerBuilder_SetTextContent(textContent) {
-      this.ctx.save();
-      var textItems = textContent.items;
-      for (var i = 0, ii = textItems.length; i < ii; i++) {
-        this.appendText(textItems[i], textContent.styles);
-      }
-      this.ctx.restore();
+        // We need to have UTF-8 encoded XML.
+        var svg_xml = unescape(encodeURIComponent(
+          (new XMLSerializer()).serializeToString(svg)));
+        var img = new Image();
+        img.src = 'data:image/svg+xml;base64,' + btoa(svg_xml);
+        img.onload = function () {
+          ctx.drawImage(img, 0, 0);
+          resolve();
+        };
+      });
+    });
+  }
+
+  return rasterizeTextLayer;
+})();
+
+/**
+ * @class
+ */
+var rasterizeAnnotationLayer = (function rasterizeAnnotationLayerClosure() {
+  var SVG_NS = 'http://www.w3.org/2000/svg';
+
+  var annotationLayerStylePromise = null;
+  function getAnnotationLayerStyle() {
+    if (annotationLayerStylePromise) {
+      return annotationLayerStylePromise;
     }
-  };
+    annotationLayerStylePromise = new Promise(function (resolve) {
+      var xhr = new XMLHttpRequest();
+      xhr.open('GET', './annotation_layer_test.css');
+      xhr.onload = function () {
+        resolve(xhr.responseText);
+      };
+      xhr.send(null);
+    });
+    return annotationLayerStylePromise;
+  }
 
-  return SimpleTextLayerBuilder;
+  function inlineAnnotationImages(images) {
+    var imagePromises = [];
+    for (var i = 0, ii = images.length; i < ii; i++) {
+      var imagePromise = new Promise(function(resolve) {
+        var xhr = new XMLHttpRequest();
+        xhr.responseType = 'blob';
+        xhr.onload = function() {
+          var reader = new FileReader();
+          reader.onloadend = function() {
+            resolve(reader.result);
+          };
+          reader.readAsDataURL(xhr.response);
+        };
+        xhr.onerror = function() {
+          resolve('');
+        };
+        xhr.open('GET', images[i].src);
+        xhr.send();
+      });
+      imagePromises.push(imagePromise);
+    }
+    return imagePromises;
+  }
+
+  function rasterizeAnnotationLayer(ctx, viewport, annotations, page) {
+    return new Promise(function (resolve) {
+      // Building SVG with size of the viewport.
+      var svg = document.createElementNS(SVG_NS, 'svg:svg');
+      svg.setAttribute('width', viewport.width + 'px');
+      svg.setAttribute('height', viewport.height + 'px');
+
+      // Adding element to host our HTML (style + annotation layer div).
+      var foreignObject = document.createElementNS(SVG_NS, 'svg:foreignObject');
+      foreignObject.setAttribute('x', '0');
+      foreignObject.setAttribute('y', '0');
+      foreignObject.setAttribute('width', viewport.width + 'px');
+      foreignObject.setAttribute('height', viewport.height + 'px');
+      var style = document.createElement('style');
+      var stylePromise = getAnnotationLayerStyle();
+      foreignObject.appendChild(style);
+      var div = document.createElement('div');
+      div.className = 'annotationLayer';
+
+      // Rendering annotation layer as HTML.
+      stylePromise.then(function (styles) {
+        style.textContent = styles;
+
+        var annotation_viewport = viewport.clone({ dontFlip: true });
+        var parameters = {
+          viewport: annotation_viewport,
+          div: div,
+          annotations: annotations,
+          page: page,
+          linkService: new LinkServiceMock()
+        };
+        PDFJS.AnnotationLayer.render(parameters);
+
+        // Inline SVG images from text annotations.
+        var images = div.getElementsByTagName('img');
+        var imagePromises = inlineAnnotationImages(images);
+        var converted = Promise.all(imagePromises).then(function(data) {
+          for (var i = 0, ii = data.length; i < ii; i++) {
+            images[i].src = data[i];
+          }
+        });
+
+        foreignObject.appendChild(div);
+        svg.appendChild(foreignObject);
+
+        // We need to have UTF-8 encoded XML.
+        converted.then(function() {
+          var svg_xml = unescape(encodeURIComponent(
+            (new XMLSerializer()).serializeToString(svg)));
+          var img = new Image();
+          img.src = 'data:image/svg+xml;base64,' + btoa(svg_xml);
+          img.onload = function () {
+            ctx.drawImage(img, 0, 0);
+            resolve();
+          };
+        });
+      });
+    });
+  }
+
+  return rasterizeAnnotationLayer;
 })();
 
 /**
@@ -118,6 +245,7 @@ var Driver = (function DriverClosure() {
     PDFJS.cMapPacked = true;
     PDFJS.cMapUrl = '../external/bcmaps/';
     PDFJS.enableStats = true;
+    PDFJS.imageResourcesPath = '/web/images/';
 
     // Set the passed options
     this.inflight = options.inflight;
@@ -153,7 +281,10 @@ var Driver = (function DriverClosure() {
 
     run: function Driver_run() {
       var self = this;
-
+      window.onerror = function(message, source, line, column, error) {
+        self._info('Error: ' + message + ' Script: ' + source + ' Line: ' +
+                  line + ' Column: ' + column + ' StackTrace: ' +  error);
+      };
       this._info('User agent: ' + navigator.userAgent);
       this._log('Harness thinks this browser is "' + this.browser +
         '" with path "' + this.appPath + '"\n');
@@ -201,8 +332,7 @@ var Driver = (function DriverClosure() {
 
       this._log('Loading file "' + task.file + '"\n');
 
-      var absoluteUrl = combineUrl(window.location.href, task.file);
-
+      var absoluteUrl = new URL(task.file, window.location).href; 
       PDFJS.disableRange = task.disableRange;
       PDFJS.disableAutoFetch = !task.enableAutoFetch;
       try {
@@ -271,6 +401,7 @@ var Driver = (function DriverClosure() {
     _nextPage: function Driver_nextPage(task, loadError) {
       var self = this;
       var failure = loadError || '';
+      var ctx;
 
       if (!task.pdfDoc) {
         var dataUrl = this.canvas.toDataURL('image/png');
@@ -301,7 +432,11 @@ var Driver = (function DriverClosure() {
         // Empty the canvas
         this.canvas.width = 1;
         this.canvas.height = 1;
-        this._clearCanvas();
+        ctx = this.canvas.getContext('2d', {alpha: false});
+        ctx.save();
+        ctx.fillStyle = 'white';
+        ctx.fillRect(0, 0, 1, 1);
+        ctx.restore();
 
         this._snapshot(task, '');
         return;
@@ -311,44 +446,88 @@ var Driver = (function DriverClosure() {
         try {
           this._log(' Loading page ' + task.pageNum + '/' +
             task.pdfDoc.numPages + '... ');
-          var ctx = this.canvas.getContext('2d');
+          this.canvas.mozOpaque = true;
+          ctx = this.canvas.getContext('2d', {alpha: false});
           task.pdfDoc.getPage(task.pageNum).then(function(page) {
             var viewport = page.getViewport(PDF_TO_CSS_UNITS);
             self.canvas.width = viewport.width;
             self.canvas.height = viewport.height;
             self._clearCanvas();
 
-            var drawContext, textLayerBuilder;
-            var resolveInitPromise;
-            var initPromise = new Promise(function (resolve) {
-              resolveInitPromise = resolve;
-            });
+            var textLayerCanvas, annotationLayerCanvas;
+            var initPromise;
             if (task.type === 'text') {
               // Using a dummy canvas for PDF context drawing operations
-              if (!self.dummyCanvas) {
-                self.dummyCanvas = document.createElement('canvas');
+              textLayerCanvas = self.textLayerCanvas;
+              if (!textLayerCanvas) {
+                textLayerCanvas = document.createElement('canvas');
+                self.textLayerCanvas = textLayerCanvas;
               }
-              drawContext = self.dummyCanvas.getContext('2d');
+              textLayerCanvas.width = viewport.width;
+              textLayerCanvas.height = viewport.height;
+              var textLayerContext = textLayerCanvas.getContext('2d');
+              textLayerContext.clearRect(0, 0,
+                textLayerCanvas.width, textLayerCanvas.height);
               // The text builder will draw its content on the test canvas
-              textLayerBuilder = new SimpleTextLayerBuilder(ctx, viewport);
-
-              page.getTextContent().then(function(textContent) {
-                textLayerBuilder.setTextContent(textContent);
-                resolveInitPromise();
-              });
+              initPromise =
+                page.getTextContent({ normalizeWhitespace: true }).then(
+                  function(textContent) {
+                    return rasterizeTextLayer(textLayerContext, viewport,
+                                              textContent);
+                });
             } else {
-              drawContext = ctx;
-              textLayerBuilder = new NullTextLayerBuilder();
-              resolveInitPromise();
+              textLayerCanvas = null;
+
+              // Render the annotation layer if necessary.
+              if (task.annotations) {
+                // Create a dummy canvas for the drawing operations.
+                annotationLayerCanvas = self.annotationLayerCanvas;
+                if (!annotationLayerCanvas) {
+                  annotationLayerCanvas = document.createElement('canvas');
+                  self.annotationLayerCanvas = annotationLayerCanvas;
+                }
+                annotationLayerCanvas.width = viewport.width;
+                annotationLayerCanvas.height = viewport.height;
+                var annotationLayerContext =
+                  annotationLayerCanvas.getContext('2d');
+                annotationLayerContext.clearRect(0, 0,
+                  annotationLayerCanvas.width, annotationLayerCanvas.height);
+
+                // The annotation builder will draw its content on the canvas.
+                initPromise =
+                  page.getAnnotations({ intent: 'display' }).then(
+                    function(annotations) {
+                      return rasterizeAnnotationLayer(annotationLayerContext,
+                                                      viewport, annotations,
+                                                      page);
+                  });
+              } else {
+                annotationLayerCanvas = null;
+                initPromise = Promise.resolve();
+              }
             }
+
             var renderContext = {
-              canvasContext: drawContext,
+              canvasContext: ctx,
               viewport: viewport
             };
             var completeRender = (function(error) {
-              page.destroy();
+              // if text layer is present, compose it on top of the page
+              if (textLayerCanvas) {
+                ctx.save();
+                ctx.globalCompositeOperation = 'screen';
+                ctx.fillStyle = 'rgb(128, 255, 128)'; // making it green
+                ctx.fillRect(0, 0, viewport.width, viewport.height);
+                ctx.restore();
+                ctx.drawImage(textLayerCanvas, 0, 0);
+              }
+              // If we have annotation layer, compose it on top of the page.
+              if (annotationLayerCanvas) {
+                ctx.drawImage(annotationLayerCanvas, 0, 0);
+              }
+              page.cleanup();
               task.stats = page.stats;
-              page.stats = new StatTimer();
+              page.stats = new pdfjsSharedUtil.StatTimer();
               self._snapshot(task, error);
             });
             initPromise.then(function () {
@@ -371,7 +550,7 @@ var Driver = (function DriverClosure() {
     },
 
     _clearCanvas: function Driver_clearCanvas() {
-      var ctx = this.canvas.getContext('2d');
+      var ctx = this.canvas.getContext('2d', {alpha: false});
       ctx.beginPath();
       ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
     },
